@@ -389,6 +389,7 @@ struct LottieGradient : LottieObject
         Array<Fill::ColorStop> output(colorStops.count + alphaCnt);
         uint32_t cidx = 0;               //color count
         uint32_t clast = colorStops.count * 4;
+        if (clast > color.input->count) clast = color.input->count;
         uint32_t aidx = clast;           //alpha count
         Fill::ColorStop cs;
 
@@ -430,7 +431,7 @@ struct LottieGradient : LottieObject
         }
 
         //color remains
-        while (cidx < clast) {
+        while (cidx + 3 < clast) {
             cs.offset = (*color.input)[cidx];
             cs.r = lroundf((*color.input)[cidx + 1] * 255.0f);
             cs.g = lroundf((*color.input)[cidx + 2] * 255.0f);
@@ -572,13 +573,15 @@ struct LottieGroup : LottieObject
     }
 
     void prepare(LottieObject::Type type = LottieObject::Group);
+    bool mergeable() override { return allowMerge; }
 
     Scene* scene = nullptr;               //tvg render data
     Array<LottieObject*> children;
 
     bool reqFragment = false;   //requirment to fragment the render context
     bool buildDone = false;     //completed in building the composition.
-    bool mergeable = true;     //if this group is consisted of simple (transformed) shapes.
+    bool allowMerge = true;     //if this group is consisted of simple (transformed) shapes.
+    bool trimpath = false;      //this group has a trimpath.
 };
 
 
@@ -595,6 +598,8 @@ struct LottieLayer : LottieGroup
         return transform->opacity(frameNo);
     }
 
+    bool mergeable() override { return false; }
+
     void prepare();
     float remap(float frameNo);
 
